package com.hsbc.hbfr.ccf.at.logreader.model;

import org.apache.log4j.Logger;

import java.util.Collection;
import java.util.LinkedList;

/**
 * Created by IntelliJ IDEA.
 * User: E9310750
 * Date: 9 juin 2005
 * Time: 14:57:08
 * To change this template use File | Settings | File Templates.
 */
public class BlockingQueue {
	private static final Logger logger = Logger.getLogger("INFRA."+BlockingQueue.class.getName());    private final Object lock = new Object();
    private final LinkedList buffer = new LinkedList();


    /**
     * Constructor that wraps (not copies).
     */
    public BlockingQueue() {
    }

    public boolean add(Object o) {
        synchronized (lock) {
			logger.debug("Blocking queue : adding 1, size = "+buffer.size());
            boolean result = buffer.add(o);
			try {
				lock.notifyAll();
			} catch (IllegalMonitorStateException e) {
				logger.error(e.getMessage(),e);
			}
			return result;
        }
    }

    public boolean addAll(Collection c) {
        synchronized (lock) {
			logger.debug("Blocking queue : adding "+c.size()+", size = "+buffer.size());
            boolean result = buffer.addAll(c);
			try {
				lock.notifyAll();
			} catch (IllegalMonitorStateException e) {
				logger.error(e.getMessage(),e);
			}
            return result;
        }
    }

    public Object get() {
        synchronized (lock) {
			logger.debug("peeking 1, size = "+buffer.size());
            while (buffer.isEmpty()) {
				logger.debug("waiting to peek");
                try {
                    lock.wait();
					logger.debug("get : notified");
                } catch (InterruptedException e) {
					logger.info(e.getMessage(), e);
                    throw new RuntimeException(e.getMessage());
                }
            }
			logger.debug("get : should be OK : "+buffer.size());
            return buffer.getFirst();
        }
    }

    public Object remove() {
        synchronized (lock) {
			logger.debug("removing 1, size = "+buffer.size());
            if (buffer.isEmpty()) {
				logger.debug("removing : waiting");
                try {
                    lock.wait(1000);
					logger.debug("remove : notified");
                } catch (InterruptedException e) {
					logger.info(e.getMessage(), e);
                    throw new RuntimeException(e.getMessage());
                }
            }
			logger.debug("remove : should be OK : "+buffer.size());
            return (buffer.size()>0)?buffer.removeFirst():null;
        }
    }

}
