package com.hsbc.hbfr.ccf.at.logreader.model;

import org.apache.log4j.Logger;

import javax.swing.*;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.util.ArrayList;
import java.util.List;

import com.hsbc.hbfr.ccf.at.logreader.ui.ProgressIndicatorNeutral;
import com.hsbc.hbfr.ccf.at.logreader.ui.MessageUtils;

/**
 * Created by IntelliJ IDEA.
 * User: E9310750
 * Date: 27 avr. 2005
 * Time: 14:47:17
 * To change this template use File | Settings | File Templates.
 */
public abstract class LogReader {
    private static final Logger logger = Logger.getLogger("INFRA."+LogReader.class.getName());
    private String currentLine = null;
    private java.io.BufferedReader reader;
    private List events = new ArrayList();
    private ProgressIndicatorNeutral progressMonitor;
    private int min;
    private int max;
    private long totalBytes;
    private long currentBytes = 0;

    public LogReader(ProgressIndicatorNeutral monitor, int min, int max, long totalBytes) {
        this.totalBytes = totalBytes;
        this.progressMonitor = monitor;
        this.min = min;
        this.max = max;
    }

    protected List parse(Reader aReader) throws IOException {
        try {
            if (logger.isInfoEnabled()) {
                Runtime r = Runtime.getRuntime();
                r.gc();

                long free = r.freeMemory();
                long total = r.totalMemory();
                long used = total-free;
                if (logger.isDebugEnabled()) {
                    logger.debug("memory before reading [used="+used+",free = "+free+",total"+total+"]");
                }
            }
            reader = new BufferedReader(aReader);
            for (SimpleLogEvent event = readEvent(); event != null; event = readEvent()) {
                addEvent(event);
            }
            if (logger.isInfoEnabled()) {
                Runtime r = Runtime.getRuntime();
                r.gc();
                long free = r.freeMemory();
                long total = r.totalMemory();
                long used = total-free;
                if (logger.isDebugEnabled()) {
                    logger.info("memory after reading [used="+used+",free = "+free+",total"+total+"]");
                }
            }
            return events;
        } finally {
            if (logger.isDebugEnabled()) {
                logger.debug("completed parsing, currentBytes = " + currentBytes + ",total =" + totalBytes);
            }
            progressMonitor.setProgress(max + 1);
            try {
                reader.close();
                reader = null;
            } catch (IOException e) {
                logger.error(MessageUtils.getErrorMessage(MessageUtils.error_closing_stream),e);
            }
        }
    }

    public abstract List parse() throws IOException;

    private void addEvent(SimpleLogEvent event) {
        events.add(event);
    }
    private final transient StringBuffer eventString = new StringBuffer();
    private SimpleLogEvent readEvent() throws IOException {
        String line;
        eventString.setLength(0);
        if (currentLine == null) {
            line = reader.readLine();
        } else {
            line = currentLine;
            currentLine = null;
        }
        if (line == null) {
            return null;
        }
        if (!isStartOfEvent(line)) {
            StringBuffer lineStarters = new StringBuffer();
            for (int i = 0; i<getEventStarters().length; i++) {
                lineStarters.append("[").append(getEventStarters()[i]).append("]");
            }
            throw new UnparseableLogFileException(MessageUtils.getErrorMessage(
                    MessageUtils.bad_start_of_event, line, lineStarters.toString()));
        }
        eventString.append(line);
        for (currentLine = reader.readLine(); currentLine != null && !isStartOfEvent(currentLine); currentLine = reader.readLine()) {
            eventString.append('\n').append(currentLine);
        }
        try {
            currentBytes += eventString.length();
            return createEvent(eventString.toString());
        } catch (UnparseableLogFileException e) {
            throw e;
        } catch (RuntimeException e) {
            String errorMessage = MessageUtils.getErrorMessage(
                    MessageUtils.unparseable_log_event, eventString.toString(), "%c;%p;%t;%d;%m");
            logger.error(errorMessage, e);
            throw new UnparseableLogFileException(errorMessage);
        } finally {
            if (totalBytes>0) {
                progressMonitor.setProgress(min + (int) ((max - min) * currentBytes / totalBytes));
            }
        }
    }

    protected abstract SimpleLogEvent createEvent(String eventString);

    private static boolean isStartOfEvent(String line) {
        for (int i =0, max = eventStarters.length; i<max; i++) {
            if (line.startsWith(eventStarters[i])) {
                return true;
            }
        }
        return false;
    }

    private static String [] eventStarters = new String[]{"INFRA.", "AUDIT.", "APPLI.", "PERF."};

    public static String[] getEventStarters() {
        return eventStarters;
    }
    public static void setEventStarters(String[] starters) {
        eventStarters = starters;
    }
}
