package com.hsbc.hbfr.ccf.at.logreader.ui;

import com.hsbc.hbfr.ccf.at.logreader.model.LogEvent;
import com.hsbc.hbfr.ccf.at.logreader.predicate.Predicate;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;

/**
 * Permet d'appliquer des filtres sur les evenements affichs
 */
public class FilteringLogEventList extends LogEventList {
    private static final Logger logger = Logger.getLogger("INFRA."+FilteringLogEventList.class.getName());

    private static class EventPredicate implements Predicate {
        private Predicate datePredicate = Predicate.TRUE;
        private Predicate priorityPredicate = Predicate.TRUE;
        private Predicate threadPredicate = Predicate.TRUE;
        private Predicate categoryPredicate = Predicate.TRUE;
        private Predicate messagePredicate = Predicate.TRUE;

        public void setDatePredicate(Predicate p) {
            datePredicate = (p == null) ? Predicate.TRUE : p;
        }

        public boolean evaluate(Object o) {
            LogEvent e = (LogEvent) o;
            return priorityPredicate.evaluate(e.getPriority())
                    && categoryPredicate.evaluate(e.getCategory())
                    && threadPredicate.evaluate(e.getThread())
                    && datePredicate.evaluate(e.getDate())
                    && messagePredicate.evaluate(e.getMessage());
        }

        public void setPriorityPredicate(Predicate priorityPredicate) {
            this.priorityPredicate = priorityPredicate;
        }

        public void setThreadPredicate(Predicate threadPredicate) {
            this.threadPredicate = threadPredicate;
        }

        public void setCategoryPredicate(Predicate categoryPredicate) {
            this.categoryPredicate = categoryPredicate;
        }

        public void fromString(String s) {

        }

        public void setMessagePredicate(Predicate messagePredicate) {
            this.messagePredicate = messagePredicate;
        }
    }

    private EventPredicate eventPredicate = new EventPredicate();
    private List notFilteredEvents;

    public FilteringLogEventList(Predicate predicate, List logEvents) {
        super(applyFilter(predicate, logEvents));
        notFilteredEvents = logEvents;
    }

    public FilteringLogEventList(List logEvents) {
        super(applyFilter(Predicate.TRUE, logEvents));
        notFilteredEvents = logEvents;
    }

    private static List applyFilter(Predicate predicate, List logEvents) {
        List newList = new ArrayList(logEvents.size());
        for (Iterator iterator = logEvents.iterator(); iterator.hasNext();) {
            Object event = iterator.next();
            if (predicate.evaluate(event)) {
                newList.add(event);
            }
        }
        logger.debug("appliedFilter ("+logEvents.size()+","+newList.size()+")");
        return newList;
    }

    public void setEvents(List events) {
        notFilteredEvents = events;
        super.setEvents(applyFilter(this.eventPredicate, notFilteredEvents));
    }

    public void setDatePredicate(Predicate p) {
        eventPredicate.setDatePredicate(p);
        applyFilter();
    }

    public void setThreadPredicate(Predicate newPred) {
        eventPredicate.setThreadPredicate(newPred);
        applyFilter();

    }

    public void setPriorityPredicate(Predicate newPred) {
        eventPredicate.setPriorityPredicate(newPred);
        applyFilter();

    }

    public void setCategoryPredicate(Predicate newPred) {
        eventPredicate.setCategoryPredicate(newPred);
        applyFilter();

    }

    public void setMessagePredicate(Predicate messagePredicate) {
        eventPredicate.setMessagePredicate(messagePredicate);
        applyFilter();

    }


    public void applyFilter() {
        setEvents(notFilteredEvents);
    }

    public List getAllEvents() {
        return notFilteredEvents;
    }
}
