package com.hsbc.hbfr.ccf.at.logreader.ui;

import com.hsbc.hbfr.ccf.at.logreader.model.*;
import com.hsbc.hbfr.ccf.at.logreader.predicate.NegatableDateAfterPredicate;
import com.hsbc.hbfr.ccf.at.logreader.predicate.NegatablePredicate;
import com.hsbc.hbfr.ccf.at.logreader.predicate.NegatableStringLikePredicate;
import org.apache.log4j.Logger;

import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.*;
import java.util.List;

public abstract class LogViewerForm {
    private static final Logger logger = Logger.getLogger("INFRA." + LogViewerForm.class.getName());
    protected static final Map globalOpenViewers = new HashMap();
    public static String currentConfig = null;
    /**
     * Permet de savoir quand la fenetre est fermee
     * utilise par le worker thread pour savoir quand se terminer
     */
    private boolean closed = false;
    /**
     * Utilis pour les progress monitors
     */
    protected static final int STEP = 100;

    protected void cleanup() {
        hiddenProviders.removeAll(hiddenProviders);
        openProviders.removeAll(openProviders);
        closed = true;
        if (globalOpenViewers.remove(getWindowObject())== null){
            logger.error("coulnd't find window object");
        }
        logger.debug("removed from globalOpenViewers(new size =" + globalOpenViewers.size() + ")");

    }
    protected abstract Object getWindowObject();
    private final BlockingQueue pendingTasks = new BlockingQueue();
    private static final Logger threadLogger = Logger.getLogger("INFRA." + LogViewerForm.class.getName()+".backgroundThread");
    private final Thread backGroundThread = new Thread(new Runnable() {
        public void run() {
            threadLogger.debug("backGround Thread started");
            while (!closed) {
                threadLogger.debug("backGround Thread. looking for task");
                Runnable task = (Runnable) pendingTasks.remove();
                threadLogger.debug("backGround Thread. obtained task");
                if (task !=null) {
                    try {
                        task.run();
                    } catch (RuntimeException e) {
                        threadLogger.error("erreur dans l'execution d'une tache asynchrone", e);
                    } catch (Error e) {
                        threadLogger.error("erreur dans l'execution d'une tache asynchrone", e);
                    }
                }
            }
            threadLogger.debug("backGround Thread terminating");
        }
    }, "LogViewerForm worker Thread");
    protected final NegatablePredicate categoryPredicate = new NegatableStringLikePredicate();
    protected final NegatablePredicate threadPredicate = new NegatableStringLikePredicate();
    protected final NegatablePredicate priorityPredicate = new NegatableStringLikePredicate();
    protected final NegatableDateAfterPredicate datePredicate = new NegatableDateAfterPredicate();
    protected final NegatablePredicate messagePredicate = new NegatableStringLikePredicate();

    protected Set openProviders = new HashSet();
    protected final Set hiddenProviders = new HashSet();
    public static final int WRAP_LIMIT = 80;

    public LogViewerForm() {
        createTableModel();
        backGroundThread.start();
    }

    protected abstract void createTableModel();

    protected abstract void initGUI();

    protected abstract void initEvents();

    public void setLogEvents(List events) {
        getLogTableModel().setEvents(events);
    }

    protected abstract LogViewerForm createInFrame(LogProvider provider);

    public void refreshProviders() {
        ProgressIndicatorNeutral monitor = createProgress(openProviders.size() * STEP);
//        ProgressMonitor monitor = new ProgressMonitor(getFrame(), "rechargement", "", 0, openProviders.size() * STEP);

        if (logger.isDebugEnabled()) {
            logger.debug("old Events size = " + getLogTableModel().getAllEvents().size());
            logger.debug("provider count : " + openProviders.size());
        }
        setLogEvents(new ArrayList());
        monitor.setProgress(0);
        Set oldOpenProviders = openProviders;
        openProviders = new HashSet();
        for (Iterator iterator = oldOpenProviders.iterator(); iterator.hasNext();) {
            LogProvider logProvider = (LogProvider) iterator.next();
            monitor.setNote(logProvider.toString());
            addFromProvider(logProvider);
            if (logger.isDebugEnabled()) {
                logger.debug("events size = " + getLogTableModel().getAllEvents().size());
            }
        }
        if (logger.isDebugEnabled()) {
            logger.debug("new Events size = " + getLogTableModel().getAllEvents().size());
        }

    }

    protected abstract FilterInput getCategoryFilter();

    protected abstract FilterInput getThreadFilter();

    protected abstract FilterInput getPriorityFilter();

    protected abstract FilterInput getDateFilter();

    protected abstract FilterInput getMessageFilter();

    /**
     * ajoute tous les evenements contenus dans le fichier aux evenements visualiss
     *
     * @param f
     */
    public void addFromFile(File f) {
        FileProvider provider = new FileProvider(f);
        addFromProvider(provider);

    }

    public void addFromURL(URL url) {
        URLLogProvider provider = new URLLogProvider(url);
        addFromProvider(provider);
    }

    public abstract PreferencesSaver.PersistentLogViewerForm getPersistentLogViewerForm();

    protected void addFromProvider(final LogProvider provider) {
        if (!openProviders.contains(provider)) {
            pendingTasks.add(createActionLoadFromProvider(provider));
        }
    }

    private Runnable createActionLoadFromProvider(LogProvider provider) {
        return new ActionLoadFromProvider(provider);
    }

    protected static String wrap(StringBuffer source) {
        int from = 0;
        do {
            String line = source.substring(from);

            int firstCR = (line.length()<WRAP_LIMIT)?line.length():line.indexOf('\n');
            if (firstCR > WRAP_LIMIT || (firstCR < 0)) {
                int targetCR = line.substring(0, WRAP_LIMIT).lastIndexOf(' ');
                if (targetCR < 0) {
                    targetCR = WRAP_LIMIT;
                }
                source.setCharAt(from + targetCR, '\n');
                from += targetCR + 1;
            } else {
                from += firstCR + 1;
            }
        } while (from <= (source.length() - WRAP_LIMIT));
        return source.toString();
    }

    protected static String wrap(String s) {
        return wrap(new StringBuffer(s));
    }

    /**
     * indique si le provider fait partie des menus MenuProvider et ColorMenu
     * @param provider
     * @return
     */
    public abstract boolean isProviderInMenu(final LogProvider provider);

    /**
     * ajoute le provider aux menus MenuProvider et ColorMenu
     * @param provider
     */
    protected abstract void addProviderToProviderMenu(final LogProvider provider);

    protected boolean isProviderHidden(LogProvider provider) {
        return hiddenProviders.contains(provider);
    }

    public void showEvents(LogProvider provider) {
        addFromProvider(provider);
        hiddenProviders.remove(provider);
    }

    protected abstract FilteringLogEventList getLogTableModel();

    /**
     * modifie les evenements visibles, en enlevant tous ceux du provider fourni
     * ne modifie pas les menus
     * todo pourrait etre implemente en utilisant un predicate, plutot qu'en detruisant tous les anciens evenements
     * @param provider
     */
    public void hideEvents(LogProvider provider) {
        List allEvents = getLogTableModel().getAllEvents();
        for (Iterator iterator = allEvents.iterator(); iterator.hasNext();) {
            LogEvent event = (LogEvent) iterator.next();
            if (event instanceof ProvidedEvent && ((ProvidedEvent) event).getProvider() == provider) {
                iterator.remove();
            }
        }
        getLogTableModel().setEvents(allEvents);
        openProviders.remove(provider);
        hiddenProviders.add(provider);
    }

    protected abstract String requestFileFromUser();

    public void readFromFile() {
        String pathname = requestFileFromUser();
        if (pathname != null) {
            File f = new File(pathname);
            if (f != null && !f.isDirectory() && f.exists()) {
                addFromFile(f);
            }
        }
    }

    protected abstract String requestURLFromUser();

    public void readFromURL() {
        try {
            String urlString = requestURLFromUser();
            if (urlString != null) {
                addFromURL(new URL(urlString));
            }
        } catch (MalformedURLException e) {
            logger.error("URL incorrecte", e);
        }
    }

    protected static String getBuildInfos() {
        return "Built on " + LogViewerForm.class.getPackage().getImplementationTitle();
    }

    // load and saving configuration and preferences */
    public void savePreferences() {
        PreferencesSaver.saveAllPreferences(getCurrentDirectory(), globalOpenViewers.values().iterator(), getColorManager().getRgbColorsByProviders());
    }

    protected abstract File getCurrentDirectory();

    public void saveConfiguration(String currentConfig) {
        PreferencesSaver.saveAllPreferences(currentConfig, getCurrentDirectory(), globalOpenViewers.values().iterator(), getColorManager().getRgbColorsByProviders());
    }
    protected abstract AbstractColorManager getColorManager();
    public Properties getConfigurations() {
        return PreferencesSaver.getConfigurations();
    }

    public void restoreConfiguration(String configName) {
        PreferencesSaver.GlobalConfig config = PreferencesSaver.restorePreferences(configName);
        setGlobalPreferences(config);
    }

    protected abstract void setBounds(int x, int y, int width, int height);

    private void restorePreferences(PreferencesSaver.PersistentLogViewerForm p) {

        Rectangle frameBounds = p.getFrameBounds();
        setBounds(frameBounds.x, frameBounds.y, frameBounds.width, frameBounds.height);
        for (Iterator iterator = p.getOpenProviders().iterator(); iterator.hasNext();) {
            LogProvider provider = (LogProvider) iterator.next();
            addFromProvider(provider);
        }
        for (Iterator iterator = p.getHiddenProviders().iterator(); iterator.hasNext();) {
            LogProvider provider = (LogProvider) iterator.next();
            ProviderColorManager.instance().chooseColorFor(provider);
            hiddenProviders.add(provider);
            addProviderToProviderMenu(provider);
        }
        restoreFilterPreferences(p.getCategoryFilter(), getCategoryFilter());
        restoreFilterPreferences(p.getThreadFilter(), getThreadFilter());
        restoreFilterPreferences(p.getPriorityFilter(), getPriorityFilter());
        restoreFilterPreferences(p.getMessageFilter(), getMessageFilter());
        restoreFilterPreferences(p.getDateFilter(), getDateFilter());

    }

    private void restoreFilterPreferences(FilterInput source, FilterInput dest) {
        for (int i = 0; i < source.getRecentTexts().length; i++) {
            String s = source.getRecentTexts()[i];
            dest.setText(s);
        }
    }

    protected void restorePreferences() {
        PreferencesSaver.GlobalConfig config = PreferencesSaver.restorePreferences();
        setGlobalPreferences(config);
    }


    protected abstract ProgressIndicatorNeutral createProgress(int max);

    private void setGlobalPreferences(PreferencesSaver.GlobalConfig config) {

        getColorManager().reinit(config.getColorsByProvider());
        int openWindowCount = config.getFormPreferences().length;
        ProgressIndicatorNeutral monitor = createProgress(openWindowCount * STEP);
        List logViewers = new ArrayList(openWindowCount);
        for (int i = 0; i < openWindowCount; i++) {
            try {
                LogViewerForm form = createInFrame(new ListLogProvider(new ArrayList()));
                if (config.getCurrentDirectory() != null) {
                    form.setCurrentDirectory(config.getCurrentDirectory());
                }
                form.restorePreferences(config.getFormPreferences()[i]);
                logViewers.add(form);
                if (logger.isDebugEnabled()) {
                    logger.debug("progress :" + ((i + 1) * STEP));
                }
                monitor.setProgress((i + 1) * STEP);
            } catch (RuntimeException e) {
                logger.error("error while restoring preferences [" + config.getFormPreferences()[i] + "]", e);
            }
        }
        for (Iterator iterator = logViewers.iterator(); iterator.hasNext();) {
            LogViewerForm logViewerForm = (LogViewerForm) iterator.next();
            logViewerForm.setVisible(true);
        }
    }

    protected abstract void setVisible(boolean b);

    protected abstract void setCurrentDirectory(File currentDirectory);


    // load and saving configuration and preferences END */
    protected abstract List getSelectedEvents();

    public String toString() {
        StringBuffer buf = new StringBuffer("providers :[");
        for (Iterator iterator = openProviders.iterator(); iterator.hasNext();) {
            LogProvider logProvider = (LogProvider) iterator.next();
            buf.append(logProvider.toString());
            if (iterator.hasNext()) {
                buf.append(";");
            }
        }
        buf.append("]");
        return buf.toString();
    }

    private class ActionLoadFromProvider implements Runnable {
        final LogProvider provider;

        ActionLoadFromProvider(final LogProvider aProvider) {
            provider = aProvider;
        }

        public void run() {
            ProgressIndicatorNeutral monitor = createProgress(STEP);
            monitor.start();

            try {
                if (logger.isDebugEnabled()) {
                    logger.debug("debug action asynchrone de chargement [ " + provider + "]");
                }
                if (openProviders.contains(provider)) {
                    return;
                }
                List events = provider.getEvents(monitor, 0, STEP);
                ProviderColorManager.instance().chooseColorFor(provider);
                ArrayList oldEvents = new ArrayList(getLogTableModel().getAllEvents());
                ArrayList newEvents = new ArrayList(oldEvents.size() + events.size());
                int readOld = 0, readNew = 0, oldCount = oldEvents.size(), newCount = events.size(), newEventCount = oldEvents.size() + events.size();
                while (readOld + readNew < newEventCount) {
                    if (readOld >= oldCount) {
                        newEvents.add(events.get(readNew++));
                    } else if (readNew >= newCount) {
                        newEvents.add(oldEvents.get(readOld++));
                    } else {

                        int order = ((LogEvent) oldEvents.get(readOld)).getDate().compareTo(((LogEvent) events.get(readNew)).getDate());
                        if (order < 0) {
                            newEvents.add(oldEvents.get(readOld++));
                        } else {
                            newEvents.add(events.get(readNew++));
                        }
                    }
                }
                openProviders.add(provider);
                addProviderToProviderMenu(provider);
                setLogEvents(newEvents);
            } catch (RuntimeException e) {
                logger.error("Une exception est survenue pendant la lecture de [" + provider + "]", e);
                openProviders.remove(provider);
            } catch (IOException e) {
                logger.error("Une exception est survenue pendant la lecture de [" + provider + "]", e);
                openProviders.remove(provider);
            } catch (OutOfMemoryError e) {
                logger.error("Pas assez de memoire pour lire [" + provider + "], fermez des fenetres ou augmentez la memoire disponible (parametre -Xmx de la ligne de commande)", e);
            } finally {
                monitor.finished();
            }

        }
    }


}
