package com.hsbc.hbfr.ccf.at.logreader.ui;

import com.hsbc.hbfr.ccf.at.logreader.model.ListLogProvider;
import com.hsbc.hbfr.ccf.at.logreader.model.LogEvent;
import com.hsbc.hbfr.ccf.at.logreader.model.LogProvider;
import com.hsbc.hbfr.ccf.at.logreader.model.ProvidedEvent;
import org.apache.log4j.Logger;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;
import java.awt.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.*;
import java.awt.event.*;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: Administrateur
 * Date: 10 juin 2005
 * Time: 22:07:49
 * To change this template use File | Settings | File Templates.
 */
public class LogViewerFormSwing extends LogViewerForm {
    private static final Logger logger = Logger.getLogger("INFRA." + LogViewerFormSwing.class.getName());

    private static final JFileChooser fc = new JFileChooser();
    private final JProgressBar progressBar = new JProgressBar();
    private static final WindowListener lastWindowCheckListener = new WindowAdapter() {
        /**
         * nettoie tout pour eviter les memory leaks
         *
         * @param e
         */
        public void windowClosed(WindowEvent e) {
            LogViewerForm closed = (LogViewerForm) globalOpenViewers.get(e.getWindow());
            closed.cleanup();
        }

        /**
         * Confirmation avant fermeture pour la sauvegarde des preferences
         *
         * @param e
         */
        public void windowClosing(WindowEvent e) {
            if (globalOpenViewers.size() == 1) {
                LogViewerFormSwing closing = (LogViewerFormSwing) globalOpenViewers.get(e.getWindow());
                closing.exitAction.actionPerformed(new ActionEvent(e.getWindow(), 0, "Exit"));
                e.getWindow().setVisible(true);
            } else {
                e.getWindow().dispose();
            }
        }
    };
    private final Action exitAction = new ExitAction();
    private final JMenu menuCouleurs = new JMenu(UIConstants.MENU_COLORS);
    private StringFilterInputPanel categoryFilter = new StringFilterInputPanel(categoryPredicate, LogEventList.COLUMN_NAMES[LogEventList.CATEGORY]);
    private StringFilterInputPanel threadFilter = new StringFilterInputPanel(threadPredicate, LogEventList.COLUMN_NAMES[LogEventList.THREAD], 12);
    private StringFilterInputPanel priorityFilter = new StringFilterInputPanel(priorityPredicate, LogEventList.COLUMN_NAMES[LogEventList.PRIORITY], 6);
    private DateFilterInputPanel dateFilter = new DateFilterInputPanel(datePredicate, LogEventList.COLUMN_NAMES[LogEventList.DATE]);
    private StringFilterInputPanel messageFilter = new StringFilterInputPanel(messagePredicate, LogEventList.COLUMN_NAMES[LogEventList.MESSAGE]);
    private FilterInput[] filters;
    private JTable table1 ;
//    = new JTable();
    private JPanel mainPanel;
    private JTextArea textArea1 = new JTextArea();
    private JMenu menuProviders = new JMenu(UIConstants.MENU_PROVIDERS);
    private MemInspector memInspector;

    protected void createTableModel() {
        table1 = new JTable();
        table1.setModel(new FilteringLogTableModel(new ArrayList()));
    }

    protected Object getWindowObject() {
        return getFrame();
    }

    public LogViewerFormSwing() {
        super();
        initGUI();
        initEvents();
    }

    protected File getCurrentDirectory() {
        return fc.getCurrentDirectory();
    }

    protected AbstractColorManager getColorManager() {
        return ProviderColorManager.instance();
    }

    protected void cleanup() {
        super.cleanup();
        JFrame frame = getFrame();
        mainPanel = null;
        menuProviders = null;
        getLogTableModel().setEvents(new ArrayList());
        memInspector.stop();
        frame.removeAll();
        logger.debug("removed from globalOpenViewers(new size =" + globalOpenViewers.size() + ")");
    }

    protected void initGUI() {
        FilteringLogEventList filteringLogTableModel = getLogTableModel();
//                new FilteringLogTableModel(new ArrayList());
        filteringLogTableModel.setCategoryPredicate(categoryPredicate);
        filteringLogTableModel.setDatePredicate(datePredicate);
        filteringLogTableModel.setPriorityPredicate(priorityPredicate);
        filteringLogTableModel.setThreadPredicate(threadPredicate);
        filteringLogTableModel.setMessagePredicate(messagePredicate);
        DefaultTableCellRenderer stringRenderer = (DefaultTableCellRenderer) table1.getDefaultRenderer(String.class);
        /**
         * permet de visualiser les fichiers d'origine des evenements, a chaque origine sa couleur de fond
         */
        table1.setDefaultRenderer(String.class, new ProvidedEventCellRenderer(this, stringRenderer));
        table1.setDefaultRenderer(Date.class, new ProvidedEventCellRenderer(this, stringRenderer) {
            SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss,SSS");

            public Component getTableCellRendererComponent(JTable table, Object value,
                                                           boolean isSelected, boolean hasFocus,
                                                           int row, int column) {
                Date d = (Date) value;
                return super.getTableCellRendererComponent(table, df.format(d), isSelected, hasFocus, row, column);
            }
        });
        table1.setDefaultRenderer(Object.class, new ProvidedEventCellRenderer(this, (DefaultTableCellRenderer) table1.getDefaultRenderer(Object.class)));
        filters = new FilterInput[LogEventList.MESSAGE + 1];
        filters[LogEventList.CATEGORY] = categoryFilter;
        filters[LogEventList.DATE] = dateFilter;
        filters[LogEventList.MESSAGE] = messageFilter;
        filters[LogEventList.PRIORITY] = priorityFilter;
        filters[LogEventList.THREAD] = threadFilter;

        JPanel panelFiltres = new JPanel(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 2, 0, 2), 0, 0);
        panelFiltres.add(categoryFilter, c);
        c.gridx++;
        panelFiltres.add(priorityFilter, c);
        c.gridx++;
        panelFiltres.add(threadFilter, c);
        c.gridx++;
        panelFiltres.add(dateFilter, c);
        c.gridx++;
        panelFiltres.add(messageFilter, c);
        mainPanel = new JPanel(new BorderLayout());
        mainPanel.add(panelFiltres, BorderLayout.NORTH);
        textArea1.setLineWrap(true);
        textArea1.setWrapStyleWord(false);
        JSplitPane split = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                new JScrollPane(table1, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER),
                new JScrollPane(textArea1, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER));
        split.setOneTouchExpandable(true);
        mainPanel.add(new JScrollPane(split, JScrollPane.VERTICAL_SCROLLBAR_NEVER, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS), BorderLayout.CENTER);

    }

    protected FilterInput getCategoryFilter() {
        return categoryFilter;
    }

    protected FilterInput getDateFilter() {
        return dateFilter;
    }

    protected FilterInput getMessageFilter() {
        return messageFilter;
    }

    protected FilterInput getPriorityFilter() {
        return priorityFilter;
    }

    protected FilterInput getThreadFilter() {
        return threadFilter;
    }

    /**
     * Installe les event listeners :
     * gestion de la selection : selectionner un evenement affiche son detail dans le text area
     * gestion du clic droit : set du filtre associ
     * configuration des combos de filtre
     */
    protected void initEvents() {
        ActionListener tableUpdater = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                getLogTableModel().applyFilter();
            }
        };
        categoryFilter.addActionListener(tableUpdater);
        messageFilter.addActionListener(tableUpdater);
        priorityFilter.addActionListener(tableUpdater);
        threadFilter.addActionListener(tableUpdater);
        dateFilter.addActionListener(tableUpdater);

        table1.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                ListSelectionModel selectionModel = table1.getSelectionModel();
                int i = selectionModel.getMaxSelectionIndex();
                List events = getLogTableModel().getEvents();
                if (i >= 0 && i < events.size()) {
                    LogEvent selEvent = (LogEvent) events.get(i);
                    if (selEvent instanceof ProvidedEvent) {
                        getFrame().setTitle(((ProvidedEvent) selEvent).getProvider().toString());
                    } else {
                        getFrame().setTitle("unknown event");
                    }
                }
            }

        });
        final int WARN_TXT_AREA_SIZE = 5000000;
        table1.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                ListSelectionModel selectionModel = table1.getSelectionModel();
                int min = selectionModel.getMinSelectionIndex();
                int max = selectionModel.getMaxSelectionIndex();
                List events = getLogTableModel().getEvents();
                StringBuffer buf = new StringBuffer();
                boolean warned = false;
                for (int i = min; i <= max; i++) {
                    if (selectionModel.isSelectedIndex(i)) {
                        LogEvent selEvent = (LogEvent) events.get(i);
                        buf.append(selEvent.getString()).append("\n");
                    }
                    if (!warned && (buf.length() > WARN_TXT_AREA_SIZE)) {
                        int option = JOptionPane.showConfirmDialog(getFrame(), "Confirmer ?", "La selection depasse " + WARN_TXT_AREA_SIZE + " caracteres, voulez vous vraiment tout afficher ?", JOptionPane.YES_NO_OPTION);
                        if (option == JOptionPane.NO_OPTION) {
                            return;
                        }
                        warned = true;
                    }
                }
                textArea1.setText(buf.toString());
                textArea1.setCaretPosition(0);

            }
        });
        table1.addMouseListener(new MouseAdapter() {
            final SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss,SSS");

            public void mouseClicked(MouseEvent e) {
                if ((e.getModifiers() & InputEvent.BUTTON3_MASK) != 0) {
// min jdk 1.4 pour ci dessous
//                if (e.getButton() == MouseEvent.BUTTON3) {
                    Point pointClicked = e.getPoint();
                    int column = table1.columnAtPoint(pointClicked);
                    int row = table1.rowAtPoint(pointClicked);
                    logger.debug("right clicked cell [" + row + "," + column + "]");
                    Object value = getLogTableModel().getValueAt(row, column);
                    if (column == LogEventList.DATE) {
                        dateFilter.setPattern(df.toPattern());
                        dateFilter.setText(df.format(value));
                    } else {
                        filters[column].setText(value.toString());
                    }
                }
            }
        });
        initDnd();
    }

    public LogViewerForm createInFrame(LogProvider provider) {
        JFrame f = new JFrame();
        f.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        final LogViewerFormSwing form = new LogViewerFormSwing();
        form.addFromProvider(provider);
        f.getContentPane().setLayout(new BorderLayout());
        f.getContentPane().add(new JScrollPane(form.mainPanel), BorderLayout.CENTER);
        JPanel statusBar = new JPanel(new BorderLayout());
        form.memInspector = new MemInspector();
        statusBar.add(form.memInspector, BorderLayout.EAST);
        statusBar.add(form.progressBar, BorderLayout.CENTER);
        form.progressBar.setVisible(false);
        f.getContentPane().add(statusBar, BorderLayout.SOUTH);
        f.setJMenuBar(form.createMenuBar());
        f.setTitle(provider.toString());
        f.setSize(800, 600);
        globalOpenViewers.put(f, form);
        f.addWindowListener(new WindowAdapter(){
            public void windowOpened(WindowEvent e) {
                form.setColumnWidths();
                form.getFrame().removeWindowListener(this);
                logger.debug("removed component listener");
            }
        });
        f.addWindowListener(lastWindowCheckListener);
        return form;
    }

    /**
     * set les tailles de colonnes pour
     * thread : taille max a 15 caracteres
     * date : taille max a la taille adequate
     * priority : taille max a la taille de DEBUG
     */
    private void setColumnWidths() {
        if (logger.isDebugEnabled()) {
            logger.debug("setting column widths on first display");
        }
        table1.setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS);
        setMaximumColumnWidth(LogEventList.PRIORITY, "DEBUG");
        setMaximumColumnWidth(LogEventList.THREAD, "THIS IS A THREA");
        setMaximumColumnWidth(LogEventList.DATE, new SimpleDateFormat("yyyy-MM-dd HH:mm:ss,SSS").format(new Date()));
    }

    private void setMaximumColumnWidth(int columnIndex, String biggestString) {
        Graphics graphics = table1.getGraphics();
        if (graphics != null) {
            FontMetrics fontMetrics = graphics.getFontMetrics();
            Rectangle2D stringBounds = fontMetrics.getStringBounds(biggestString, graphics);
            int OFFSET = 5;
            TableColumn column = table1.getColumnModel().getColumn(columnIndex);
            int preferredWidth = (int) (stringBounds.getWidth() + OFFSET);
            column.setPreferredWidth(preferredWidth);
            column.setMaxWidth(preferredWidth);
        }
    }

    protected void addProviderToProviderMenu(final LogProvider provider) {
        if (provider.isTransient() || isProviderInMenu(provider)) {
            return;
        }

        if (logger.isDebugEnabled()) {
            logger.debug("adding color to menu for [" + provider + "]");
        }
        JMenuItem item = new ChangeProviderColorMenuItem(provider);
        menuCouleurs.add(item);
        item = new ToggleProviderMenuItem(provider);
        item.setSelected(!isProviderHidden(provider));
        item.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                ToggleProviderMenuItem menuItem = (ToggleProviderMenuItem) e.getSource();

                if (e.getStateChange() != ItemEvent.SELECTED) {
                    hideEvents(menuItem.provider);
                } else {
                    showEvents(menuItem.provider);
                }
            }
        });

        menuProviders.add(item);


    }

    protected String requestURLFromUser() {
        return (String) JOptionPane.showInputDialog(getFrame(), "entrez l'url (avec login et password si necessaire)", "Saisie", JOptionPane.QUESTION_MESSAGE, null, null, "ftp://user01:pass1234@ftp.foo.com/README.txt");
    }

    protected String requestFileFromUser() {
        int returnVal = fc.showOpenDialog(null);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            return fc.getSelectedFile().getAbsolutePath();
        } else {
            return null;
        }
    }

    public boolean isProviderInMenu(final LogProvider provider) {
        String s = provider.toString();
        for (int i = 0, max = menuCouleurs.getItemCount(); i < max; i++) {
            if (menuCouleurs.getItem(i).getText().equals(s)) {
                return true;
            }
        }
        return false;
    }

    protected FilteringLogEventList getLogTableModel() {
        return (FilteringLogEventList) table1.getModel();
    }

    private void initDnd() {
        DropTarget dropTarget = new DropTarget(this.mainPanel, DnDConstants.ACTION_COPY, new DropTargetListener() {
            public void drop(DropTargetDropEvent dtde) {
                if (logger.isDebugEnabled()) {
                    logger.debug("drop");
                }
                if (dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
                    dtde.acceptDrop(DnDConstants.ACTION_REFERENCE);
                    try {
                        List transferData = (List) dtde.getTransferable().getTransferData(DataFlavor.javaFileListFlavor);
                        for (Iterator iterator = transferData.iterator(); iterator.hasNext();) {
                            File file = (File) iterator.next();
                            addFromFile(file);
                        }
                        dtde.dropComplete(true);
                    } catch (UnsupportedFlavorException e) {
                        dtde.rejectDrop();
                        logger.error("error while accepting drop", e);
                    } catch (IOException e) {
                        dtde.rejectDrop();
                        logger.error("error while accepting drop", e);
                    }
                } else {
                    dtde.rejectDrop();
                }
            }

            public void dragEnter(DropTargetDragEvent dtde) {
                if (logger.isDebugEnabled()) {
                    DataFlavor[] currentDataFlavors = dtde.getCurrentDataFlavors();

                    for (int i = 0; i < currentDataFlavors.length; i++) {
                        DataFlavor flavor = currentDataFlavors[i];
                        logger.debug("flavor : " + flavor.getHumanPresentableName() + "/" + flavor.getMimeType());
                    }

                }

                if (dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
                    dtde.acceptDrag(DnDConstants.ACTION_REFERENCE);
                } else {
                    dtde.rejectDrag();
                }

            }

            public void dragOver(DropTargetDragEvent dtde) {
            }

            public void dropActionChanged(DropTargetDragEvent dtde) {
            }

            public void dragExit(DropTargetEvent dte) {
            }
        });


    }

    private void setFrameTitle(String title) {
        getFrame().setTitle(title);
    }
    private static class ActionOpenFile extends AbstractAction {
         LogViewerForm form;

         public ActionOpenFile(LogViewerForm form) {
             super(UIConstants.MENU_FILE_OPENFILE);
             this.form = form;
         }

         public void actionPerformed(ActionEvent event) {
             form.readFromFile();
         }
     }
    private JMenuBar createMenuBar() {
        JMenuBar mb = new JMenuBar();
        JMenu fileMenu = new JMenu(UIConstants.MENU_FILE);
        fileMenu.add(new ActionOpenFile(this));
        fileMenu.add(new ActionOpenURL(this));
        fileMenu.add(new ActionRefreshEvents(this));
        fileMenu.addSeparator();
        fileMenu.add(new AbstractAction(UIConstants.MENU_FILE_CLOSE) {
            public void actionPerformed(ActionEvent e) {
                getFrame().dispose();
            }
        });
        fileMenu.addSeparator();
        fileMenu.add(new AbstractAction(UIConstants.MENU_FILE_CONFIGURATION_LOAD) {
            public void actionPerformed(ActionEvent e) {
                Properties props = getConfigurations();
                if (props.containsKey("configuration.next")) {
                    props.remove("configuration.next");
                }
                List configsSorted = new ArrayList(props.values());
                Collections.sort(configsSorted);
                String sel = (String) JOptionPane.showInputDialog(getFrame(), "Quelle configuration ?", "Charger", JOptionPane.QUESTION_MESSAGE, null, configsSorted.toArray(), null);
                if (sel != null) {
                    currentConfig = sel;
                    for (Iterator iterator = globalOpenViewers.values().iterator(); iterator.hasNext();) {
                        LogViewerFormSwing form = (LogViewerFormSwing) iterator.next();
                        form.getFrame().dispose();
                    }

                    restoreConfiguration(sel);
                }
            }

        });

        fileMenu.add(new AbstractAction(UIConstants.MENU_FILE_CONFIGURATION_SAVE) {
            public void actionPerformed(ActionEvent e) {
                if (currentConfig == null) {
                    currentConfig = JOptionPane.showInputDialog("choisissez un nom");
                }
                if (currentConfig != null) {
                    saveConfiguration(currentConfig);
                }
            }

        });
        fileMenu.add(new AbstractAction(UIConstants.MENU_FILE_CONFIGURATION_SAVEAS) {
            public void actionPerformed(ActionEvent e) {
                currentConfig = JOptionPane.showInputDialog("choisissez un nom");
                if (currentConfig != null) {
                    saveConfiguration(currentConfig);
                }
            }
        });

        fileMenu.addSeparator();
        fileMenu.add(exitAction);
        mb.add(fileMenu);
        mb.add(menuCouleurs);
        mb.add(menuProviders);
        mb.add(new JMenuItem(new SpawnLogViewerAction(this)));
        mb.add(Box.createHorizontalGlue());
        JMenu menuAPropos = new JMenu("?");
        menuAPropos.add(new JMenuItem(new AbstractAction("Aide") {
            public void actionPerformed(ActionEvent event) {
                showHelp();
            }
        }));
        menuAPropos.addSeparator();

        menuAPropos.add(new JMenuItem(new AbstractAction("Infos") {
            public void actionPerformed(ActionEvent event) {
                JOptionPane.showMessageDialog(getFrame(), getBuildInfos(), "Informations", JOptionPane.INFORMATION_MESSAGE);
            }
        }));

        mb.add(menuAPropos);
        return mb;
    }

    protected void setCurrentDirectory(File currentDirectory) {
        fc.setCurrentDirectory(currentDirectory);
    }

    protected void setVisible(boolean b) {
        getFrame().setVisible(b);
    }

    protected void setBounds(int x, int y, int width, int height) {
        getFrame().setBounds(x, y, width, height);
    }


    private JFrame getFrame() {
        return (table1== null)?null:(JFrame) table1.getTopLevelAncestor();
    }

    protected List getSelectedEvents() {
        LogEventList model = getLogTableModel();
        ListSelectionModel selectionModel = table1.getSelectionModel();
        int from = selectionModel.getMinSelectionIndex();
        int to = table1.getSelectionModel().getMaxSelectionIndex();
        List l = new ArrayList();
        List events = model.getEvents();
        for (int i = from; i <= to; i++) {
            if (selectionModel.isSelectedIndex(i)) {
                l.add(events.get(i));
            }
        }
        return l;
    }

    public static void showHelp() {
        try {
            URL urlReadme = LogViewerForm.class.getResource("/readme.html");
            JEditorPane pane = new JEditorPane(urlReadme);
            JFrame f = new JFrame("Aide");
            f.getContentPane().add(new JScrollPane(pane));
            f.setSize(800, 400);
            f.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
            f.setVisible(true);
        } catch (IOException e) {
            logger.error("Impossible de trouver ou lire le fichier d'aide", e);
        }

    }

    private class ExitAction extends AbstractAction {
        public ExitAction() {
            super(UIConstants.MENU_FILE_QUIT);
        }

        public void actionPerformed(ActionEvent e) {
            int i = JOptionPane.showConfirmDialog(getFrame(), "Voulez vous conserver cette configuration pour la prochaine fois ?");
            switch (i) {
                case JOptionPane.YES_OPTION:
                    savePreferences();
                case JOptionPane.NO_OPTION:
                    System.exit(0);
                    break;
                case JOptionPane.CANCEL_OPTION:
                    return;
                default :
            }
        }
    }

    private static class ChangeProviderColorAction extends AbstractAction {
        LogViewerFormSwing form;
        LogProvider provider;

        public ChangeProviderColorAction(LogViewerFormSwing logViewerForm, LogProvider provider) {
            super(provider.toString());
            this.form = logViewerForm;
            this.provider = provider;
        }

        public void actionPerformed(ActionEvent e) {
            Color newColor = JColorChooser.showDialog(null, provider.toString(), ProviderColorManager.instance().getColorForProvider(provider));
            if (newColor != null) {
                ProviderColorManager.instance().setColorForProvider(provider, newColor);
                form.table1.repaint();
            }
        }
    }

    private static class ActionRefreshEvents extends AbstractAction {
        LogViewerForm form;

        public ActionRefreshEvents(LogViewerForm form) {
            super(UIConstants.MENU_FILE_REFRESH);
            this.form = form;
        }

        public void actionPerformed(ActionEvent event) {
            form.refreshProviders();
        }
    }


    private static class ActionOpenURL extends AbstractAction {
        LogViewerForm form;

        public ActionOpenURL(LogViewerForm form) {
            super(UIConstants.MENU_FILE_OPENURL);
            this.form = form;
        }

        public void actionPerformed(ActionEvent event) {
            form.readFromURL();
        }
    }

    private static class SpawnLogViewerAction extends AbstractAction {
        LogViewerForm form;

        public SpawnLogViewerAction(LogViewerForm f) {
            super(UIConstants.MENU_SPAWN);
            form = f;

        }

        public void actionPerformed(ActionEvent event) {
            List l = form.getSelectedEvents();
            form.createInFrame(new ListLogProvider(l)).setVisible(true);

        }
    }

    private static class ProvidedEventCellRenderer implements TableCellRenderer {
        LogViewerForm form;
        private DefaultTableCellRenderer renderer;

        public ProvidedEventCellRenderer(LogViewerForm f, DefaultTableCellRenderer renderer) {
            this.renderer = renderer;
            form = f;
        }

        public Component getTableCellRendererComponent(JTable table, Object value,
                                                       boolean isSelected, boolean hasFocus,
                                                       int row, int column) {

            LogEvent event = (LogEvent) form.getLogTableModel().getEvents().get(row);
            renderer.setBackground(ProviderColorManager.instance().getColorForEvent(event));
            return renderer.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
        }
    }

    private class ChangeProviderColorMenuItem extends JMenuItem {
        private final LogProvider provider;

        public ChangeProviderColorMenuItem(LogProvider provider) {
            super(new ChangeProviderColorAction(LogViewerFormSwing.this, provider));
            this.provider = provider;
        }

        public Color getForeground() {
            return ProviderColorManager.instance().getColorForProvider(provider);
        }
    }

    private static class ToggleProviderMenuItem extends JCheckBoxMenuItem {
        private final LogProvider provider;

        public ToggleProviderMenuItem(LogProvider provider) {
            super(provider.toString());
            this.provider = provider;
        }

        public Color getForeground() {
            return ProviderColorManager.instance().getColorForProvider(provider);
        }
    }

    public PreferencesSaver.PersistentLogViewerForm getPersistentLogViewerForm() {
        PersistentLogViewerFormImpl p = new PersistentLogViewerFormImpl();
        p.openProviders = openProviders;
        p.hiddenProviders = hiddenProviders;
        p.frameBounds = getFrame().getBounds();
        p.categoryFilter = categoryFilter;
        p.threadFilter = threadFilter;
        p.priorityFilter = priorityFilter;
        p.messageFilter = messageFilter;
        p.dateFilter = dateFilter;
        return p;
    }

    protected ProgressIndicatorNeutral createProgress(int max) {
        if (getFrame() == null || !getFrame().isVisible()){
            final ProgressMonitor aMonitor = new ProgressMonitor(null, "Veuillez patienter pendant le rechargement des logs", "", 0, max);
            aMonitor.setMillisToDecideToPopup(1);
            aMonitor.setMillisToPopup(1);
            return new ProgressIndicatorNeutral() {
                ProgressMonitor monitor = aMonitor;
                public int getMax() {
                    return monitor.getMaximum();
                }

                public void setNote(String s) {
                    monitor.setNote(s);
                }

                public void setMax(int max) {
                    monitor.setMaximum(max);
                }

                public void start() {
                    progressBar.setVisible(true);
                }

                public void setProgress(int progress) {
                    monitor.setProgress(progress);
                }

                public int getProgress() {
                    return 0;  //To change body of implemented methods use File | Settings | File Templates.
                }

                public void finished() {
                    //To change body of implemented methods use File | Settings | File Templates.
                }
            };
        } else {
            return new  ProgressIndicatorNeutral() {
                public void finished() {
                    progressBar.setValue(progressBar.getMaximum());
                    progressBar.setVisible(false);
                }

                public int getMax() {
                    return progressBar.getMaximum();  //To change body of implemented methods use File | Settings | File Templates.
                }

                public int getProgress() {
                    return progressBar.getValue();  //To change body of implemented methods use File | Settings | File Templates.
                }

                public void setMax(int max) {
                    progressBar.setMaximum(max);
                }

                public void setNote(String s) {
                    progressBar.setToolTipText(s);
                }

                public void setProgress(int progress) {
                    progressBar.setValue(progress);
                }

                public void start() {

                }
            };
        }
    }

    public static void main
            (String[] args) {
        URL url = LogViewerForm.class.getResource("/log4j.properties");
//		System.out.println("trying to initialize log4j using [" + url + "]");
        org.apache.log4j.PropertyConfigurator.configure(url);
        logger.info("STARTING NEW SESSION : " + getBuildInfos());
        LogViewerFormSwing model = new LogViewerFormSwing();
        model.restorePreferences();
        if (globalOpenViewers.size() == 0) {
            JFrame f = ((LogViewerFormSwing) model.createInFrame(new ListLogProvider(new ArrayList()))).getFrame();
            f.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    ((LogViewerForm) globalOpenViewers.values().iterator().next()).savePreferences();
                    System.exit(0);
                }
            });
            f.setVisible(true);
        }
    }
}
