package com.hsbc.hbfr.ccf.at.logreader.ui;

import com.hsbc.hbfr.ccf.at.logreader.model.LogProvider;
import com.hsbc.hbfr.ccf.at.logreader.logproviders.LogProviderFactory;

import java.util.*;
import java.util.List;
import java.awt.*;
import java.io.*;

import org.apache.log4j.Logger;

/**
 * Created by IntelliJ IDEA.
 * User: Administrateur
 * Date: 10 juin 2005
 * Time: 11:47:06
 * To change this template use File | Settings | File Templates.
 */
public class PreferencesSaver {
    private static final String COLOR_PREFIX = "colors.";

   public static interface GlobalConfig {
        PersistentLogViewerForm[] getFormPreferences();
        File getCurrentDirectory();
        Map getColorsByProvider();
    }
    private static class GlobalConfigImpl implements GlobalConfig {
        private File currentDir;
        private PersistentLogViewerForm[] forms =new PersistentLogViewerForm[0];
        private final Map colorsByProvider = new HashMap();

        private GlobalConfigImpl() {
        }

        public File getCurrentDirectory() {
            return currentDir;
        }

        public PersistentLogViewerForm[] getFormPreferences() {
            return forms;
        }

        public Map getColorsByProvider() {
            return colorsByProvider;
        }
    }
    public static interface PersistentLogViewerForm {
        Set getOpenProviders();
        Set getHiddenProviders();
        Rectangle getFrameBounds();
        FilterInput getCategoryFilter();
        FilterInput getThreadFilter();
        FilterInput getPriorityFilter();
        FilterInput getMessageFilter();
        FilterInput getDateFilter();
    }
    private static final Logger logger = Logger.getLogger("INFRA."+PreferencesSaver.class);
    private static final String PREFERENCES_DIR = "dir";
    private final PersistentLogViewerForm form;

    public PreferencesSaver(PersistentLogViewerForm o) {
        form = o;
    }
    private static File getPrefsDir() {
        String userHome = System.getProperty("user.home");
        File userHomeFile = new File(userHome);
        File prefsDir = new File(userHomeFile, ".logViewer");
        if (!prefsDir.exists()) {
            prefsDir.mkdir();
            logger.info("created preferences directory [" + prefsDir.getPath() + "]");
        }
        return prefsDir;
    }
    public static void saveAllPreferences(File currentDir, Iterator allViewers, Map rgbColorsByProviders) {
        savePreferences(createGlobalPreferences(currentDir, allViewers, rgbColorsByProviders));
    }

    /**
     * cree le properties a sauver a partir des elements de configuration passes en parametres
     * @param currentDir
     * @param allViewers
     * @param rgbColorsByProvider
     * @return
     */
    private static Properties createGlobalPreferences(File currentDir, Iterator allViewers, Map rgbColorsByProvider) {
        Properties preferences = new Properties();
        preferences.put(PREFERENCES_DIR, currentDir.getPath());
        Iterator it = rgbColorsByProvider.entrySet().iterator();
        logger.debug("BEGIN saving color preferences");
        while (it.hasNext()) {
            Map.Entry entry = (Map.Entry) it.next();
            LogProvider provider = (LogProvider) entry.getKey();
            String colorRgb = (String) entry.getValue();
            preferences.setProperty("colors." + provider.toString(), "" + colorRgb);
        }
        logger.debug("END saving color preferences");
        int viewerCount = 0;
        logger.debug("BEGIN saving window preferences");
        for (Iterator iterator = allViewers; iterator.hasNext(); viewerCount++) {
            LogViewerForm form = (LogViewerForm) iterator.next();
            String prefix = "openWindows." + viewerCount;
            logger.debug("BEGIN saving window preferences(" + viewerCount + ")");
            new PreferencesSaver(form.getPersistentLogViewerForm()).storePreferences(preferences, prefix);
            logger.debug("END saving window preferences(" + viewerCount + ")");
        }
        preferences.put("openWindows.count", "" + viewerCount);
        logger.debug("BEGIN saving window preferences");
        return preferences;
    }

    public static void savePreferences(Properties preferences) {
        try {
            File prefsDir = getPrefsDir();
            File prefsFile = new File(prefsDir, "preferences.properties");
            FileOutputStream fos = new FileOutputStream(prefsFile);
            try {
                preferences.store(fos, "");
            } finally {
                fos.close();
            }
        } catch (IOException e) {
            logger.error("Couldn't save preferences to [" + getPrefsDir() + "/preferences.properties]", e);
        } catch (RuntimeException e) {
            logger.error("Couldn't save preferences to [" + getPrefsDir() + "/preferences.properties]", e);
        }

    }

    /**
     * renvoie vrai si des properties ont ete ajoutees
     *
     * @param props
     * @param prefix
     * @return
     */
    private boolean storePreferences(Properties props, String prefix) {
        boolean openProvidersStored = storeProviders(props, prefix + ".provider", form.getOpenProviders());
        boolean hiddenProvidersStored = storeProviders(props, prefix + ".hiddenProvider", form.getHiddenProviders());
        if (openProvidersStored || hiddenProvidersStored) {

            if (form.getFrameBounds() != null) {
                Rectangle frameBounds = form.getFrameBounds();
                props.setProperty(prefix + ".bounds.x", "" + frameBounds.x);
                props.setProperty(prefix + ".bounds.y", "" + frameBounds.y);
                props.setProperty(prefix + ".bounds.width", "" + frameBounds.width);
                props.setProperty(prefix + ".bounds.height", "" + frameBounds.height);
            }
            storeFilterPreferences(props, prefix);
        }
        return (openProvidersStored || hiddenProvidersStored);
    }

    /**
     * @param props
     * @param prefix    ends with .provider or .hiddenProvider
     * @param providers
     * @return true if any information has been persisted, false otherwise
     */
    private boolean storeProviders(Properties props, String prefix, Set providers) {
        int j = 0;
        for (Iterator itProviders = providers.iterator(); itProviders.hasNext();) {
            LogProvider provider = (LogProvider) itProviders.next();
            if (!provider.isTransient()) {
                props.put(prefix + "." + j, provider.toString());
                j++;
            }
        }
        if (j > 0) {
            props.setProperty(prefix + "Count", "" + j);
        }
        return (j > 0);
    }

    private void storeFilterPreferences(Properties props, String prefix) {
        storeFilterPreferences(props, prefix + ".filters.category", form.getCategoryFilter());
        storeFilterPreferences(props, prefix + ".filters.thread", form.getThreadFilter());
        storeFilterPreferences(props, prefix + ".filters.priority", form.getPriorityFilter());
        storeFilterPreferences(props, prefix + ".filters.message", form.getMessageFilter());
        storeFilterPreferences(props, prefix + ".filters.date", form.getDateFilter());
    }
    private void storeFilterPreferences(Properties props, String prefix, FilterInput model) {
        String[] options = model.getRecentTexts();
        props.setProperty(prefix + ".count", "" + options.length);

        for (int i = 0, max = options.length; i < max; i++) {
            props.setProperty(prefix + "." + i, options[i]);
        }

        if (model.getText() != null) {
            props.setProperty(prefix + ".selected", "" + model.getText());
        }
    }
    /**
     * sauve la configuration actuelle en l'associant au nom configName
     *
     * @param configName
     * @param rgbColorsByProviders
     */
    public static void saveAllPreferences(String configName, File currentDir, Iterator allViewers, Map rgbColorsByProviders) {
        Properties configs = getConfigurations();
        boolean isNewConfig = !configs.containsValue(configName);
        Properties config = createGlobalPreferences(currentDir, allViewers, rgbColorsByProviders);
        String configFileName = null;
        if (isNewConfig) {
            configFileName = createNewConfigFilename(configs);
        } else {
            for (Iterator iterator = configs.entrySet().iterator(); iterator.hasNext();) {
                Map.Entry entry = (Map.Entry) iterator.next();
                if (configName.equals(entry.getValue())) {
                    configFileName = (String) entry.getKey();
                    break;
                }
            }
        }
        File prefsFile = new File(getPrefsDir(), configFileName + ".properties");
        try {

            FileOutputStream fos = new FileOutputStream(prefsFile);
            try {
                config.store(fos, "");
            } finally {
                fos.close();
            }
            configs.setProperty(configFileName, configName);
            saveConfigurations(configs);
        } catch (IOException e) {
            logger.error("Couldn't save preferences to [" + prefsFile + "]", e);
        } catch (RuntimeException e) {
            logger.error("Couldn't save preferences to [" + prefsFile + "]", e);
        }
    }

    private static void saveConfigurations(Properties configs) {
        FileOutputStream fos = null;
        File configsFile = new File(getPrefsDir(), "configurations.properties");
        try {
            fos = new FileOutputStream(configsFile);
            try {
                configs.store(fos, "");
            } finally {
                fos.close();
            }
        } catch (FileNotFoundException e) {
            logger.error("Couldn't save configurations to [" + configsFile + "]", e);
        } catch (IOException e) {
            logger.error("Couldn't save configurations to [" + configsFile + "]", e);
        }
    }

    /**
     * creer un novueau nom pour stocker une novuelle config
     * Ce nom ne doit pas encore etre utilis
     *
     * @param configs la liste des config existante : filename=>config name
     * @return a relative filename
     */
    private static String createNewConfigFilename(Properties configs) {
        String nextString;
        if (configs.containsKey("configuration.next")) {
            nextString = configs.getProperty("configuration.next");
        } else {
            nextString = "0";
        }
        int next = Integer.parseInt(nextString);
        String configName = "config" + (next++);
        configs.setProperty("configuration.next", "" + next);
        return configName;
    }

    public static Properties getConfigurations() {
        try {
            File prefsDir = getPrefsDir();
            File configurationsFile = new File(prefsDir, "configurations.properties");
            Properties config = new Properties();
            if (configurationsFile.exists()) {
                FileInputStream inStream = new FileInputStream(configurationsFile);
                try {
                    config.load(inStream);
                } finally {
                    inStream.close();
                }
            }
            return config;
        } catch (IOException e) {
            logger.error(e);
            return new Properties();
        }
    }
    public static GlobalConfig restorePreferences(String configName) {
        GlobalConfigImpl config = new GlobalConfigImpl();
        Properties configs = getConfigurations();
        String configFileName = null;
        for (Iterator iterator = configs.entrySet().iterator(); iterator.hasNext();) {
            Map.Entry entry = (Map.Entry) iterator.next();
            if (configName.equals(entry.getValue())) {
                configFileName = (String) entry.getKey();
                break;
            }
        }
        if (configFileName == null) {
            return config;
        }
        File prefsDir = getPrefsDir();
        Properties preferences = new Properties();
        File prefsFile = new File(prefsDir, configFileName + ".properties");
        try {
            FileInputStream fis = new FileInputStream(prefsFile);
            try {
                preferences.load(fis);
            } finally {
                fis.close();
            }
        } catch (IOException e) {
            logger.error("Couldn't load preferences from [" + prefsDir + "/preferences.properties]", e);
        } catch (RuntimeException e) {
            logger.error("Couldn't load preferences from [" + prefsDir + "/preferences.properties]", e);
        }
        return getGlobalPreferences(preferences);
    }


    public static GlobalConfig restorePreferences() {
        File prefsDir = getPrefsDir();
        Properties preferences = new Properties();
        File prefsFile = new File(prefsDir, "preferences.properties");
        if (!prefsFile.exists()) {
            return new GlobalConfigImpl();
        }
        try {
            FileInputStream fis = new FileInputStream(prefsFile);
            try {
                preferences.load(fis);
            } finally {
                fis.close();
            }
        } catch (IOException e) {
            logger.error("Couldn't load preferences from [" + prefsDir + "/preferences.properties]", e);
        } catch (RuntimeException e) {
            logger.error("Couldn't load preferences from [" + prefsDir + "/preferences.properties]", e);
        }
        return getGlobalPreferences(preferences);
    }

    /**
     * Recree une config a partir de preferences persistees
     * @param preferences
     * @return
     */
    private static GlobalConfig getGlobalPreferences(Properties preferences) {
        GlobalConfigImpl global = new GlobalConfigImpl();
        if (preferences.getProperty(PREFERENCES_DIR) != null) {
            global.currentDir= new File(preferences.getProperty(PREFERENCES_DIR));
            if (logger.isDebugEnabled()){
                logger.debug("previous currentDir : "+global.currentDir.getAbsolutePath());
            }
        }

        if (preferences.getProperty("openWindows.count") != null) {
            int openWindowCount = Integer.parseInt(preferences.getProperty("openWindows.count"));
            if (openWindowCount > 0) {
                global.forms = new PersistentLogViewerForm[openWindowCount];
                for (int i = 0; i < openWindowCount; i++) {
                    PersistentLogViewerFormImpl persistent= new  PersistentLogViewerFormImpl();
                    restorePreferences(preferences, "openWindows." + i, persistent);
                    global.forms[i] = persistent;
                }
            }
        }
        for (Enumeration enum = preferences.propertyNames(); enum.hasMoreElements();) {
            String prop =(String) enum.nextElement();
            if (prop.startsWith(COLOR_PREFIX)) {

                String providerString = prop.substring(COLOR_PREFIX.length());
                String colorString = preferences.getProperty(prop);
                if (logger.isDebugEnabled()) {
                    logger.debug("restoring color for ["+providerString+"] = ["+colorString+"]");
                }
                global.colorsByProvider.put(providerString, colorString);
            }
        }

        return global;
    }
    private static FilterInput createFilterInput(Properties props, String prefix) {
        FilterInputImpl model = new FilterInputImpl();
        String s = props.getProperty(prefix + ".count");
        if (s != null) {
            int count = Integer.parseInt(s);
            for (int i = 0; i < count; i++) {
                model.setText(props.getProperty(prefix + "." + i));
            }
        }
        String sel = props.getProperty(prefix + ".selected");
        if (sel == null) {
            model.setText("");
        } else {
            model.setText(sel);
        }
        return model;
    }
    private static List getProviders(Properties prefs, String prefix) {
        String count = prefs.getProperty(prefix + "Count");
        int providerCount = (count == null) ? 0 : Integer.parseInt(count);
        List l = new ArrayList();
        for (int j = 0; j < providerCount; j++) {
            l.add(LogProviderFactory.createProvider(prefs.getProperty(prefix + "." + j)));
        }
        return l;

    }

    private static void restorePreferences(Properties props, String prefix, PersistentLogViewerFormImpl p)
    {
        if (props.getProperty(prefix + ".bounds.x") != null) {
            p.frameBounds = new Rectangle(
                    Integer.parseInt(props.getProperty(prefix + ".bounds.x")),
                    Integer.parseInt(props.getProperty(prefix + ".bounds.y")),
                    Integer.parseInt(props.getProperty(prefix + ".bounds.width")),
                    Integer.parseInt(props.getProperty(prefix + ".bounds.height")));

        }
        p.openProviders =new HashSet(getProviders(props, prefix + ".provider"));
        p.hiddenProviders =new HashSet(getProviders(props, prefix + ".hiddenProvider"));
        p.categoryFilter= createFilterInput(props, prefix + ".filters.category");
        p.threadFilter= createFilterInput(props, prefix + ".filters.thread");
        p.priorityFilter= createFilterInput(props, prefix + ".filters.priority");
        p.messageFilter= createFilterInput(props, prefix + ".filters.message");
        p.dateFilter= createFilterInput(props, prefix + ".filters.date");
    }


}
