package com.hsbc.hbfr.ccf.at.logreader.ui;

import com.hsbc.hbfr.ccf.at.logreader.model.LogEvent;
import com.hsbc.hbfr.ccf.at.logreader.model.LogProvider;
import com.hsbc.hbfr.ccf.at.logreader.model.ProvidedEvent;
import org.apache.log4j.Logger;

import javax.swing.*;
import java.awt.*;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: E9310750
 * Date: 12 mai 2005
 * Time: 16:52:13
 * To change this template use File | Settings | File Templates.
 */
public class ProviderColorManager extends AbstractColorManager{
    private static final ProviderColorManager INSTANCE = new ProviderColorManager();
    private static final Logger logger = Logger.getLogger("INFRA." + ProviderColorManager.class.getName());
    private static final Random colorRandomizer = new Random();
    private static final JTable dummyTable = new JTable();
    private final Map colorsByProvider = new HashMap();
    /**
     * key : provider.toString value : color.rgbValue()
     */
    private final Map uninterpretedColorsByProvider = new HashMap();
//	private final List listeners = new ArrayList();

    private ProviderColorManager() {
    }

    public static ProviderColorManager instance() {
        return INSTANCE;
    }

    /**
     * @param provider
     * @return the color to be used for the given provider. returns the table background if no specific color is registered
     */
    public Color getColorForProvider(LogProvider provider) {
        return chooseColorFor(provider);
//        Object o = colorsByProvider.get(provider);
//        return (o == null) ? dummyTable.getBackground() : (Color) o;
    }

    /**
     * associates the given provider and color
     * triggers repaint of the table
     *
     * @param provider
     * @param newColor
     */
    public void setColorForProvider(LogProvider provider, Color newColor) {
        if (logger.isDebugEnabled()) {
            logger.debug("setting color for [" + provider + "] = " + newColor);
        }
        boolean isNew = colorsByProvider.get(provider) == null;
        colorsByProvider.put(provider, newColor);
        if (isNew) {
            fireProviderAdded(provider, newColor);
        } else {
            fireProviderColorChanged(provider, newColor);
        }
//		fireProviderAdded(provider);
        // todo fire colorChangedEvent for provider
//		table1.invalidate();
//		table1.repaint();
    }

    /**
     * @param aMap key : provider.toString value : color.rgbValue()
     */
    public void reinit(Map aMap) {
        if (logger.isDebugEnabled()) {
            logger.debug("reinit");
        }
        this.colorsByProvider.clear();
        uninterpretedColorsByProvider.clear();
        for (Iterator iterator = aMap.entrySet().iterator(); iterator.hasNext();) {
            Map.Entry entry = (Map.Entry) iterator.next();
            uninterpretedColorsByProvider.put((String) entry.getKey(), (String) entry.getValue());
        }
    }

    protected void fireProviderColorChanged(LogProvider provider, Color newColor) {
//		ProviderColorEvent event = new ProviderColorEvent(this, provider, newColor, ProviderColorEvent.PROVIDER_COLOR_CHANGED);
//		fire(event);
    }

    protected void fireProviderAdded(LogProvider provider, Color newColor) {
//		ProviderColorEvent event = new ProviderColorEvent(this, provider, newColor, ProviderColorEvent.PROVIDER_ADDED);
//		fire(event);
    }
/*
	protected void fire(ProviderColorEvent event) {
		for (Iterator iterator = listeners.iterator(); iterator.hasNext();) {
			ProviderListener providerListener = (ProviderListener) iterator.next();
			try {
				providerListener.providerAdded(event);
			} catch (Exception e) {
				logger.error("error while dispatching event",e);
			}
		}
	}
  */

    /**
     * @param event
     * @return the color to be used to display the event
     */
    public Color getColorForEvent(LogEvent event) {
        if (event instanceof ProvidedEvent) {
            return getColorForProvider(((ProvidedEvent) event).getProvider());
        } else {
            return dummyTable.getBackground();
        }
    }

    /**
     * if the provider is already associated with a color use it,
     * else if the provider is transient (result of a selection) use background color
     * else chooses a color at random
     *
     * @param provider
     */
    public Color chooseColorFor(LogProvider provider) {
        Object alreadyChosenColor = colorsByProvider.get(provider);
        if (provider != null && !provider.isTransient() && alreadyChosenColor == null) {
            String prefColor = (String) uninterpretedColorsByProvider.get(provider.toString());
            Color newColor = (prefColor == null) ? getRandomColor() : new Color(Integer.parseInt(prefColor));
            setColorForProvider(provider, newColor);
            return newColor;
        }
        return (Color) alreadyChosenColor;
    }

    /**
     * choose a color at random
     *
     * @return
     */
    private static Color getRandomColor() {
        return new Color(colorRandomizer.nextInt(256), colorRandomizer.nextInt(256), colorRandomizer.nextInt(256));
    }

    public Set getKnownProviders() {
        return colorsByProvider.keySet();
    }

/*
	public void addProviderListener(ProviderListener l) {
		if (!listeners.contains(l)) {
			listeners.add(l);
		}
	}
	public void removeProviderListener(ProviderListener l) {
		if (listeners.contains(l)) {
			listeners.remove(l);
		}
	}       */

    /**
     * return a map associating providers and the associated color RGB value as a stringified int.
     * the int is made of (RED>>16 & 0xFF) | (GREEN >>8 & 0xFF) | (BLUE >>0 & 0xFF)
     *
     * @return
     */
    protected Map getRgbColorsByProviders() {
        Map ret = new HashMap();
        Iterator it = getKnownProviders().iterator();
        while (it.hasNext()) {
            LogProvider provider = (LogProvider) it.next();
            Color c = getColorForProvider(provider);
            ret.put(provider, "" + c.getRGB());
        }
        return ret;
    }
}
